@@ -16,6 +16,9 @@
 #include <signal.h>
 #include <errno.h>
 #include <sys/types.h>
+#ifdef SYSV
+#include <unistd.h>
+#endif
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
@@ -244,7 +247,7 @@ void setViewportColormap(disp, scrn, visual)
     cmap_windows[0]= ImageWindow;
     cmap_windows[1]= ViewportWin;
     XChangeProperty(disp, ViewportWin, cmap_atom, XA_WINDOW, 32,
-		    PropModePrepend, cmap_windows, 2);
+		    PropModePrepend, (unsigned char *) cmap_windows, 2);
   }
 
 }
@@ -269,7 +272,7 @@ static char *iconName(s)
    * want to change this.
    */
 
-  if (t= rindex(buf, '/')) {
+  if ((t = rindex(buf, '/'))) {
     for (s= buf, t++; *t; s++, t++)
       *s= *t;
     *s= '\0';
@@ -287,14 +290,14 @@ static struct visual_class_name {
   int   class; /* numerical value of class */
   char *name;  /* actual name of class */
 } VisualClassName[] = {
-  TrueColor,   "TrueColor",
-  DirectColor, "DirectColor",
-  PseudoColor, "PseudoColor",
-  StaticColor, "StaticColor",
-  GrayScale,   "GrayScale",
-  StaticGray,  "StaticGray",
-  StaticGray,  "StaticGrey",
-  -1,          NULL
+  { TrueColor,   "TrueColor" },
+  { DirectColor, "DirectColor" },
+  { PseudoColor, "PseudoColor" },
+  { StaticColor, "StaticColor" },
+  { GrayScale,   "GrayScale" },
+  { StaticGray,  "StaticGray" },
+  { StaticGray,  "StaticGrey" },
+  { -1,          NULL }
 };
 
 int visualClassFromName(name)
@@ -377,7 +380,7 @@ void bestVisual(disp, scrn, image, rvisual, rdepth)
      unsigned int  *rdepth;
 { unsigned int  depth, a;
   Screen       *screen;
-  Visual       *visual, *default_visual;
+  Visual       *visual=NULL, *default_visual;
 
   /* figure out the best depth the server supports.  note that some servers
    * (such as the HP 11.3 server) actually say they support some depths but
@@ -541,7 +544,7 @@ char imageInWindow(disp, scrn, image, user_geometry, fullscreen, install,
   Visual               *visual;
   unsigned int          depth;
   Window                oldimagewindow;
-  Colormap              oldcmap;
+  Colormap              oldcmap=None;
   XSetWindowAttributes  swa_img;
   XSetWindowAttributes  swa_view;
   XClassHint            classhint;
@@ -549,7 +552,7 @@ char imageInWindow(disp, scrn, image, user_geometry, fullscreen, install,
   XSizeHints            sh;
   XWMHints              wmh;
   int                   pixx= -1, pixy= -1;
-  int                   lastx, lasty, mousex, mousey;
+  int                   lastx=0, lasty, mousex, mousey;
   int                   paint;
   static int            old_width= -1, old_height= -1;
   static Atom           proto_atom= None, delete_atom= None;
@@ -684,7 +687,7 @@ char imageInWindow(disp, scrn, image, user_geometry, fullscreen, install,
     delete_atom = XInternAtom(disp, "WM_DELETE_WINDOW", False);
     if ((proto_atom != None) && (delete_atom != None))
       XChangeProperty(disp, ViewportWin, proto_atom, XA_ATOM, 32,
-		      PropModePrepend, &delete_atom, 1);
+		      PropModePrepend, (unsigned char *) &delete_atom, 1);
    paint= 0;
   }
   else {
@@ -921,7 +924,7 @@ char imageInWindow(disp, scrn, image, user_geometry, fullscreen, install,
       mousex= event.button.x;
       mousey= event.button.y;
       /*XSync(disp, False); */
-      while (XCheckTypedEvent(disp, MotionNotify, &event) == True) {
+      while (XCheckTypedEvent(disp, MotionNotify, (XEvent *) &event) == True) {
 	mousex= event.button.x;
 	mousey= event.button.y;
       }
