@@ -26,12 +26,17 @@
  * SUCH DAMAGE.
  */
 /*
- * $Id: mgp.c,v 1.87 1998/09/06 03:39:25 itojun Exp $
+ * $Id: mgp.c,v 1.122 2001/09/27 04:00:55 kazu Exp $
  */
 
-static char *mgp_version = "1.04a (980906)";
+static char *mgp_version = "1.09a (20010927)";
 
 #include "mgp.h"
+#ifdef USE_SETLOCALE
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
+#endif
 #include <fcntl.h>
 #ifdef TTY_KEYINPUT
 #include <termios.h>
@@ -73,12 +78,17 @@ char *vfcap_name = VFCAP;
 #ifdef FREETYPE
 char *freetypefontdir = FREETYPEFONTDIR;
 char *freetypefont0 = NULL;	/* font name to be used as a last resort */
+char *freetypemfont0 = NULL;	/* font name to be used as a last resort */
+
+int unicode_map[65536];
 #endif
+int latin_unicode_map[3][256]; /* currently we have iso8859-2-4 map */
 
 static char *tsfile = NULL;
 static char *dumpdir = NULL;
 
 char *gsdevice = DEFAULT_GSDEV;
+char *htmlimage;
 
 static time_t srctimestamp;
 
@@ -176,6 +186,8 @@ cleanup(int sig)
 		sigprocmask(SIG_UNBLOCK, &mask, NULL);
 		kill(mypid, sig);
 	}
+
+	finish_win(); /* finish X connection */
 	exit(-sig);
 }
 
@@ -187,22 +199,26 @@ main(argc, argv)
 	int opt;
 	extern char *optarg;
 	extern int optind;
-	u_char *progname;
-	u_int start_page = 1;
-	char buf[BUFSIZ], *p;
+	char *progname;
+	int start_page = 1;
+	char buf[BUFSIZ], *p, *p2;
 	extern char *Paths[];
 	extern int NumPaths;
 #ifdef VFLIB
 	char *vfont_name = VFONT;
 #endif
+	char *ep;
 
+#ifdef USE_SETLOCALE
+	setlocale(LC_CTYPE, "");
+#endif
 	progname = argv[0];
 
 	if ((p = getenv("MGPWDIR")) != NULL) {
 		mgpwdir = p;
 	}
 
-#define ACCEPTOPTS	"BdvVob:c:g:f:hlGp:qt:Q:PST:D:CORw:X:x:n"
+#define ACCEPTOPTS	"BdvVob:c:eg:f:hlGp:qt:Q:PST:D:CORw:X:x:nF:E:"
 	while ((opt = getopt(argc, argv, ACCEPTOPTS)) != -1) {
 #undef ACCEPTOPTS
 		switch (opt) {
@@ -246,7 +262,16 @@ main(argc, argv)
 			break;
 
 		case 'p':
+			ep = NULL;
 			start_page = atoi(optarg);
+			if (start_page <= 0) {
+				mgp_usage(progname);
+				/*NOTREACHED*/
+			}
+			break;
+
+		case 'e':
+			mgp_flag |= FL_GLYPHEDGE;
 			break;
 
 		case 'g':
@@ -264,7 +289,7 @@ main(argc, argv)
 			break;
 
 		case 'Q':
-			b_quality = atoi(optarg);
+			b_quality[caching] = atoi(optarg);
 			quality_flag = 1;
 			break;
 
@@ -323,6 +348,22 @@ main(argc, argv)
 		case 'n':
 			mgp_flag |= FL_NOSTDIN;
 			break;
+
+		case 'F':
+			mgp_flag |= FL_FRDCACHE;
+			p = optarg;
+			if ((p2 = strsep(&p, ",")))
+				cache_mode = atoi(p2);
+			if ((p2 = strsep(&p, ",")))
+				cache_effect = atoi(p2);
+			if ((p2 = strsep(&p, ",")))
+				cache_value = atoi(p2);
+			break;
+
+		case 'E':
+			htmlimage = optarg;
+			break;
+
 		case 'h':
 		default:
 			mgp_usage(progname);
@@ -390,7 +431,8 @@ main(argc, argv)
 	}
 
 	cleanup(0);	/* never returns */
-	/* NOTREACHED */
+	exit(0);	/* avoid warnings */
+	/*NOTREACHED*/
 }
 
 static void
@@ -404,6 +446,22 @@ genhtml(start_page)
 	FILE *txt;
 	int page;
 	char *childdebug;
+	char convdb[][3][16] = {{ "jpg", "cjpeg", "djpeg" },
+				{ "png", "pnmtopng", "pngtopnm" },
+				{ NULL, NULL, NULL }};
+	int inum = 0;
+
+	/* check image type */
+	if (htmlimage) {
+		for (inum = 0; *convdb[inum] != NULL; inum++) {
+			if (strcmp(*convdb[inum], htmlimage) == 0)
+				break;
+		}
+		if (*convdb[inum] == NULL) {
+			fprintf(stderr, "unknwon image type %s.\n", htmlimage);
+			cleanup(-1);
+		}
+	}
 
 	/* check if we can write to the directory */
 	sprintf(buf, "%s/%ld", dumpdir, (long)time((time_t *)NULL));
@@ -425,21 +483,22 @@ genhtml(start_page)
 		state_goto(&state, page, 0);
 		draw_page(&state, NULL);
 
+
+#define EXT convdb[inum][0]
+
 		/*
-		 * dump out gif image
+		 * dump out image
 		 */
 		fprintf(stderr, "(full image)");
 		sprintf(buf, "xwintoppm -silent -name MagicPoint | "
-			"ppmquant 256 %s | "
-			"ppmtogif %s > %s/mgp%05d.gif",
-			childdebug, childdebug, dumpdir, page);
+			"%s %s > %s/mgp%05d.%s",
+			convdb[inum][1], childdebug, dumpdir, page, EXT);
 		system(buf);	/*XXX security hole*/
 		fprintf(stderr, "(thumbnail)");
-		sprintf(buf, "giftopnm %s/mgp%05d.gif | "
-			"pnmscale 0.25 | ppmquant 256 %s | "
-			"ppmtogif %s > %s/mgp%05d.idx.gif",
-			dumpdir, page, childdebug, childdebug,
-			dumpdir, page);
+		sprintf(buf, "%s %s/mgp%05d.%s | "
+			"pnmscale 0.25 | %s %s > %s/mgp%05d.idx.%s",
+			convdb[inum][2], dumpdir, page, EXT, convdb[inum][1], childdebug,
+			dumpdir, page, EXT);
 		system(buf);	/*XXX security hole*/
 
 		/*
@@ -474,13 +533,13 @@ genhtml(start_page)
 		fprintf(html, "<BR>Page %d: %s<BR>\n", page, page_title(page));
 		fprintf(html, "<HR>\n");
 		if (window_width < 0 || window_height < 0) {
-			fprintf(html, "<IMG SRC=\"mgp%05d.gif\" "
+			fprintf(html, "<IMG SRC=\"mgp%05d.%s\" "
 				"ALT=\"Page %d\">\n",
-				page, page);
+				page, EXT, page);
 		} else {
-			fprintf(html, "<IMG SRC=\"mgp%05d.gif\" "
+			fprintf(html, "<IMG SRC=\"mgp%05d.%s\" "
 				"WIDTH=%d HEIGHT=%d ALT=\"Page %d\"><BR>\n",
-				page, window_width, window_height,
+				page, EXT, window_width, window_height,
 				page);
 		}
 		fprintf(html, "<HR>Generated by "
@@ -539,15 +598,15 @@ txtdone:;
 	for (page = start_page; page <= maxpage; page++) {
 		if (window_width < 0 || window_height < 0) {
 			fprintf(html, "<A HREF=\"mgp%05d.html\">"
-				"<IMG SRC=\"mgp%05d.idx.gif\" "
+				"<IMG SRC=\"mgp%05d.idx.%s\" "
 				"ALT=\"Page %d\"></A>\n",
-				page, page, page);
+				page, page, EXT, page);
 		} else {
 			fprintf(html, "<A HREF=\"mgp%05d.html\">"
-				"<IMG SRC=\"mgp%05d.idx.gif\" "
+				"<IMG SRC=\"mgp%05d.idx.%s\" "
 				"WIDTH=%d HEIGHT=%d "
 				"ALT=\"Page %d\"></A>\n",
-				page, page, window_width / 4,
+				page, page, EXT, window_width / 4,
 				window_height / 4, page);
 		}
 	}
@@ -578,31 +637,42 @@ mgp_usage(name)
 	char *name;
 {
 	fprintf(stderr, "Usage: %s [opts] mgpfile\n", name);
-	fprintf(stderr, "\t-B: Ignore background image\n");
-	fprintf(stderr, "\t-d: Demo mode - go through the presentation\n");
-	fprintf(stderr, "\t-V: Be verbose\n");
-	fprintf(stderr, "\t-v: Show version number and quit\n");
-	fprintf(stderr, "\t-o: Do not override the window manager\n");
-	fprintf(stderr, "\t-G: Page guide is on\n");
+
+	fprintf(stderr, "\t-b <color>: Specify background color\n");
 #ifdef VFLIB
-	fprintf(stderr, "\t-l: Do not use outline font on vflib\n");
 	fprintf(stderr, "\t-c <vfcap>: Specify vfontcap file\n");
+#endif /*VFLIB*/
+	fprintf(stderr, "\t-d: Demo mode - go through the presentation\n");
+#ifdef VFLIB
 	fprintf(stderr, "\t-f <font>: Specify default vflib font\n");
 #endif /*VFLIB*/
-	fprintf(stderr, "\t-x <engine>: Disable specified rendering engine\n");
-	fprintf(stderr, "\t-b <color>: Specify background color\n");
-	fprintf(stderr, "\t-p <page>: Start at the specified page\n");
 	fprintf(stderr, "\t-g <geometry>: Set window geometry\n");
-	fprintf(stderr, "\t-Q <quality>: Set background image quality(0-100)\n");
+	fprintf(stderr, "\t-h Display this help message\n");
+#ifdef VFLIB
+	fprintf(stderr, "\t-l: Do not use outline font on vflib\n");
+#endif /*VFLIB*/
+	fprintf(stderr, "\t-n: Disables control key input from tty\n");
+	fprintf(stderr, "\t-o: Do not override the window manager\n");
+	fprintf(stderr, "\t-p <page>: Start at the specified page\n");
 	fprintf(stderr, "\t-q Do not beep on errors\n");
 	fprintf(stderr, "\t-t <timeslot>: Enable presentation timer\n");
+	fprintf(stderr, "\t-v: Show version number and quit\n");
+	fprintf(stderr, "\t-w <dir>: Specify a working directory\n");
+	fprintf(stderr, "\t-x <engine>: Disable specified rendering engine\n");
+
+	fprintf(stderr, "\t-B: Ignore background image\n");
+	fprintf(stderr, "\t-C: Use private colormap\n");
+	fprintf(stderr, "\t-D <dir>: Generate html pages for the presentation\n");
+	fprintf(stderr, "\t-F<mode>,<effect>,<value>: Use forwarding caches\n");
+	fprintf(stderr, "\t-G: Page guide is on\n");
+	fprintf(stderr, "\t-O: Obey to the window manager\n");
+	fprintf(stderr, "\t-Q <quality>: Set background image quality(0-100)\n");
+	fprintf(stderr, "\t-R: Do not perform automatic reload\n");
 	fprintf(stderr, "\t-S: Do not process directives that forks process\n");
 	fprintf(stderr, "\t-T <timestampfile>: Update timestampfile on page refresh\n");
-	fprintf(stderr, "\t-D <dir>: Generate html pages for the presentation\n");
-	fprintf(stderr, "\t-C: Use private colormap\n");
-	fprintf(stderr, "\t-R: Do not perform automatic reload\n");
+	fprintf(stderr, "\t-V: Be verbose\n");
 	fprintf(stderr, "\t-X <gsdevice>: ghostscript device to use\n");
-	fprintf(stderr, "\t-n: Disables control key input from tty\n");
+
 	exit(0);
 }
 
@@ -639,12 +709,12 @@ main_loop(start_page)
 #if 0
 	/* be conservative about first page... */
 	draw_page(&state, NULL);
+#endif
 	if (pg_mode) {
 		pg_on();
 		pg_draw(&state);
 		XFlush(display);
 	}
-#endif
 
 #ifdef TTY_KEYINPUT
 	if (!(mgp_flag & FL_NOSTDIN)) {
@@ -662,13 +732,19 @@ main_loop(start_page)
 			XDefineCursor(display, window, pen_curs);
 		else
 			XUndefineCursor(display, window);
+
 		do {
 			; /*nothing*/
 		} while (draw_one(&state, &e) == False);
 
 		if (t_fin)
 			timebar(&state);
+#if 0 /* last page reload bug fix? */
 		if (state.cp && state.cp->ct_op == CTL_PAUSE) {
+#else
+		if ((state.cp && state.cp->ct_op == CTL_PAUSE) 
+			|| (state.page == maxpage)){ 
+#endif 
 			if (tsfile) {
 				int fd;
 				fd = open(tsfile, O_WRONLY|O_TRUNC|O_CREAT,
@@ -688,6 +764,7 @@ main_loop(start_page)
 			}
 
 			if (wantreload()) {
+				draw_reinit(&state);
 				cleanup_file();
 				load_file(mgp_fname);
 				if (maxpage < state.page)
@@ -697,6 +774,7 @@ main_loop(start_page)
 		}
 
 		prevpage = state.page;
+
 		switch (e.type) {
 		case EnterNotify:
 			for (i = 1; i <= maxpage; i++) {
@@ -793,6 +871,7 @@ main_loop(start_page)
 			case XK_j:
 			case XK_n:
 			case XK_Down:
+			case XK_Next:
 			case XK_space:
 			    {
 				struct render_state tstate;
@@ -800,12 +879,11 @@ main_loop(start_page)
 
 				if (number == 0)
 					number = 1;
-				if (state.cp && state.cp->ct_op == CTL_PAUSE)
+				if (state.cp && state.cp->ct_op == CTL_PAUSE){
 					state_next(&tstate);
-				else if (state.page + number
+				} else if (state.page + number
 						<= maxpage) {
-					state_goto(&tstate,
-						state.page + number, 0);
+						state_goto(&tstate, state.page + number, 0);
 				} else {
 					beep();
 					break;
@@ -826,6 +904,7 @@ main_loop(start_page)
 			case XK_k:
 			case XK_p:
 			case XK_Up:
+			case XK_Prior:
 			case XK_BackSpace:
 			case XK_Delete:
 				if (number == 0) number = 1;
@@ -887,6 +966,9 @@ rakugaki_toggle:
 				if (control) {
 reload:
 					pl_off(&state);
+					cached_page = 0;
+					reset_background_pixmap();
+					draw_reinit(&state);
 					cleanup_file();
 					load_file(mgp_fname);
 					if (maxpage < state.page)
@@ -922,6 +1004,8 @@ repaint:;
 						number = maxpage;
 					if (number <= maxpage && number > 0) {
 						state_goto(&state, number, 0);
+						state_newpage(&state);
+						state_init(&state);
 					} else
 						beep();
 				}
@@ -941,6 +1025,21 @@ repaint:;
 				number = number * 10 + key - XK_0;
 				break;
 
+			case XK_c:
+				if (verbose) 
+					if (mgp_flag & FL_FRDCACHE)
+						printf("turn off forward cache\n");
+					else			
+						printf("turn on forward cache\n");
+
+				mgp_flag ^= FL_FRDCACHE;
+			break;
+
+			case XK_a:
+				XCopyArea(display, cachewin, window, gc_cache,
+						0, 0, window_width, window_height, 0, 0);
+				break;
+
 			default:
 				number = 0;
 				break;
@@ -950,6 +1049,7 @@ repaint:;
 
 		case KeyRelease:
 			key = XLookupKeysym((XKeyEvent *)&e, 0);
+
 			switch (key) {
 			case XK_Control_L:
 			case XK_Control_R:
@@ -987,12 +1087,18 @@ repaint:;
 			break;
 
 		case ConfigureNotify:
-			if (e.xconfigure.window != window)
+			if ((e.xconfigure.window != window) && 
+				((mgp_flag & FL_OVER) || 
+				e.xconfigure.window != RootWindow(display, screen)))
 				break;
 			if (window_width != e.xconfigure.width
 			 || window_height != e.xconfigure.height) {
 				struct ctrl *lastcp;
 
+				if (!(mgp_flag & FL_OVER))
+					XMoveResizeWindow(display, window, 0, 0, 
+						e.xconfigure.width, e.xconfigure.height);
+
 				if (pg_mode)
 					pg_off();
 				pl_off(&state);
@@ -1000,10 +1106,16 @@ repaint:;
 				window_height = e.xconfigure.height;
 				state.width = e.xconfigure.width;
 				state.height = e.xconfigure.height;
-				draw_sizechange(&state);	/*notify*/
+				if (mgp_flag & FL_FRDCACHE) {
+					cached_page = 0;
+					reset_background_pixmap();
+				}
+				reset_background_pixmap();
+				draw_reinit(&state);	/*notify*/
 				lastcp = state.cp;
 				state_goto(&state, state.page, 1);
 				draw_page(&state, lastcp);
+
 			}
 			if (pg_mode) {
 				pg_on();
@@ -1045,6 +1157,7 @@ rakugaki_update(state, e)
 		XButtonPressedEvent *eb;
 		eb = (XButtonPressedEvent *)e;
 		x = eb->x; y = eb->y;
+		if (e->xbutton.button != 1) return;
 	} else {
 		rakugaki_x = rakugaki_y = -1;
 		return;
@@ -1212,3 +1325,22 @@ wantreload()
 
 	return 0;
 }
+
+/*
+  remap child window which was invoked by xsystem directive.
+  this is an adhoc solution for window-maker.
+*/
+void
+remapchild()
+{
+	int	i;
+
+	for (i = 0; i < childidx; i++) {
+		if (childtab[i].window_id > 0){
+			XMapSubwindows(display, window);
+			XFlush(display);
+			return;
+		}
+	}
+	return;
+}
