@@ -26,7 +26,7 @@
  * SUCH DAMAGE.
  */
 /*
- * $Id: mgp.h,v 1.94 1998/09/04 21:21:42 itojun Exp $
+ * $Id: mgp.h,v 1.128 2001/09/17 14:48:30 nishida Exp $
  */
 
 #include <stdio.h>
@@ -101,13 +101,12 @@
 #define PAGELIST_KFONT	"k14"
 
 #define DEFAULT_CHARSIZE	10	/* 10% of height */
-#define DEFAULT_HGAP		10
+#define DEFAULT_HGAP		0
 #define DEFAULT_VGAP		15
 #define DEFAULT_BQUALITY	100
 
-#define KANJIFONT_FORMAT \
-			"-*-*-medium-r-*-*-%s-*-*-*-*-*-%s"
-#define FONT_FORMAT	"-*-%s-*-*-%s-*-*-*-*-*-iso8859-1"
+#define XLFD_HYPHEN	14
+#define FONT_FORMAT	"-*-%s-*-*-%s-*-*-*-*-*-%s"
 #define DEFAULT_X_FONT	"times-medium-r"
 #define CUTIN_DELAY	15000
 #define SHRINK_DELAY	00500
@@ -118,11 +117,15 @@
 #define RCFILE		".mgprc"
 
 #define	EMBEDDIR	"EMBEDDIR/"
+#ifndef UUDECODE
 #define	UUDECODE	"/usr/bin/uudecode"
+#endif
+#ifndef GUNZIP
 #define	GUNZIP		"/usr/contrib/bin/gunzip"
+#endif
 
-#define MAXPAGE		200
-#define MAXLINE		100
+#define MAXPAGE		512
+#define MAXLINE		256
 #define MAXVALLEN	512
 #define MAXDIREC	16
 #define MAXARG		32
@@ -130,6 +133,8 @@
 #define MAXSTYLE	100
 #define MAXFONTDEF	100
 
+#define MAXBGPIXMAP	2
+
 #define SP_NONE		0
 #define SP_SHRINK	1
 #define SP_LCUTIN	2
@@ -148,19 +153,21 @@
 #define	DEFAULT_MGPWDIR		"/tmp"
 
 /* 	mgp command line flags 	*/
-#define FL_OVER		0x01
-#define FL_BIMAGE	0x02
-#define FL_DEMO		0x04
-#define FL_VERBOSE	0x08
-#define FL_OUTLINE	0x10
-#define FL_NOBEEP	0x20
-#define FL_NOFORK	0x80
-#define FL_PRIVATE	0x100
-#define FL_NODECORATION	0x200
-#define FL_NOAUTORELOAD	0x400
-#define FL_NOVFLIB	0x800
+#define FL_OVER		0x0001
+#define FL_BIMAGE	0x0002
+#define FL_DEMO		0x0004
+#define FL_VERBOSE	0x0008
+#define FL_OUTLINE	0x0010
+#define FL_NOBEEP	0x0020
+#define FL_NOFORK	0x0080
+#define FL_PRIVATE	0x0100
+#define FL_NODECORATION	0x0200
+#define FL_NOAUTORELOAD	0x0400
+#define FL_NOVFLIB	0x0800
 #define FL_NOFREETYPE	0x1000
 #define	FL_NOSTDIN	0x2000
+#define	FL_GLYPHEDGE	0x4000
+#define	FL_FRDCACHE		0x8000
 
 /* 	page attribute flags 	*/
 #define PGFLAG_NODEF	0x01	/* nodefault */
@@ -186,21 +193,34 @@ struct ctrl_long {
 };
 
 struct ctrl_char {
-	u_char *ct_value;
+	char *ct_value;
+};
+
+struct ctrl_char2 {
+	char *ct_value1;
+	char *ct_value2;
 };
 
 struct ctrl_args {
 	u_int ct_argc;
-	u_char **ct_argv;
+	char **ct_argv;
 	int ct_flag;
 };
 
 struct ctrl_image {
-	u_char *ct_fname;
+	char *ct_fname;
 	u_int ct_numcolor;
 	u_int ct_ximagesize;
 	u_int ct_yimagesize;
 	u_int ct_zoomflag;
+#define Z_XMASK		0x0f
+#define Z_YMASK		0xf0
+#define Z_YSHIFT	4
+#define Z_ABSOLUTE	0x00	/* absolute value */
+#define Z_NORMAL	0x01	/* normal zoom ratio */
+#define Z_SCREEN	0x02	/* screen relative zoom ratio */
+#define Z_SCREEN0	0x03	/* original screen size specified */
+#define Z_OBEY		0x04	/* obey other axis */
 };
 
 /* for gradation image generation*/
@@ -232,11 +252,25 @@ struct ctrl_bar {
 };
 
 struct ctrl_icon {
-	u_char *ct_value; 
+	char *ct_value; 
 	u_long ct_color; 
 	u_int ct_size;   
 };  
 
+struct ctrl_area {
+	u_int ct_xoff;
+	u_int ct_width;
+	u_int ct_yoff;
+	u_int ct_height;
+};
+
+struct ctrl_pcache {
+	u_int ct_cflag;
+	u_int ct_cmode;
+	u_int ct_ceffect;
+	u_int ct_cvalue;
+};
+
 struct ctrl {
 	u_char ct_op;
 	u_char ct_flag;
@@ -247,11 +281,14 @@ struct ctrl {
 		struct ctrl_int ctrl_int;
 		struct ctrl_long ctrl_long;
 		struct ctrl_char ctrl_char;
+		struct ctrl_char2 ctrl_char2;
 		struct ctrl_image ctrl_image;
 		struct ctrl_grad ctrl_grad;
 		struct ctrl_bar ctrl_bar;
 		struct ctrl_args ctrl_args;
 		struct ctrl_icon ctrl_icon;
+		struct ctrl_area ctrl_area;
+		struct ctrl_pcache ctrl_pcache;
 	} ct_val;
 };
 
@@ -259,6 +296,8 @@ struct ctrl {
 #define cti_value	ct_val.ctrl_int.ct_value
 #define ctl_value	ct_val.ctrl_long.ct_value
 #define ctc_value	ct_val.ctrl_char.ct_value
+#define ctc2_value1	ct_val.ctrl_char2.ct_value1
+#define ctc2_value2	ct_val.ctrl_char2.ct_value2
 #define ctm_fname	ct_val.ctrl_image.ct_fname
 #define ctm_numcolor	ct_val.ctrl_image.ct_numcolor
 #define ctm_ximagesize	ct_val.ctrl_image.ct_ximagesize
@@ -286,13 +325,23 @@ struct ctrl {
 #define ctic_value	ct_val.ctrl_icon.ct_value
 #define ctic_color	ct_val.ctrl_icon.ct_color
 #define ctic_size	ct_val.ctrl_icon.ct_size
+#define	ctar_xoff	ct_val.ctrl_area.ct_xoff
+#define	ctar_width	ct_val.ctrl_area.ct_width
+#define	ctar_yoff	ct_val.ctrl_area.ct_yoff
+#define	ctar_height	ct_val.ctrl_area.ct_height
+#define ctch_flag	ct_val.ctrl_pcache.ct_cflag
+#define ctch_mode	ct_val.ctrl_pcache.ct_cmode
+#define ctch_effect	ct_val.ctrl_pcache.ct_ceffect
+#define ctch_value	ct_val.ctrl_pcache.ct_cvalue
 
 struct ctl_words {
         u_char ctl_type;
 	char ctl_vtype;
 #define T_STR	'c'
+#define T_STR2	'C'
 #define T_INT	'i'
 #define T_LONG	'l'
+#define T_DOUBLE 'f'
 #define T_SP	'x'
 #define T_VOID	'-'
         char *ctl_string;
@@ -306,7 +355,16 @@ struct page_attribute {
 	u_int pg_linenum;
 	u_int pg_b_numcolor;	/* background gradation number of colors */
 	u_int pg_b_dir;		/* background gradation deg */
-} page_attribute[MAXPAGE];
+	u_int pg_text;		/* this page is text only */
+};
+extern struct page_attribute page_attribute[MAXPAGE];
+
+extern int	caching;
+extern int	cached_page;
+extern int	cache_hit;
+extern int	cache_mode;
+extern int	cache_effect;
+extern int	cache_value;
 
 struct render_state {
 	/* state of the parser */
@@ -319,8 +377,8 @@ struct render_state {
 		 * DEFAULT - doing default_control
 		 * PAGE    - doing page_control
 		 */
-	u_char *curprefix;
-	u_char *tabprefix;
+	char *curprefix;
+	char *tabprefix;
 	u_int align;
 	u_int special;
 	u_int leftfillpos;
@@ -333,6 +391,10 @@ struct render_state {
 	Drawable target;
 	u_int height;
 	u_int width;
+	int xprefix;
+	int tabxprefix;
+	int xoff;
+	int yoff;
 	u_int ypos;
 	int have_mark;
 	u_int mark_ypos;
@@ -340,7 +402,8 @@ struct render_state {
 	int maxascent;
 	int maxdescent;
 	u_int linewidth;
-	u_char *xfont;
+	u_int brankline;
+	struct ctrl *xfont;
 	struct render_object *obj, *objlast;
 };
 
@@ -364,6 +427,9 @@ struct render_object {
 #endif /* FREETYPE */
 		O_XFONT,
 		O_IMAGE,
+#ifdef MNG
+		O_ANIM,
+#endif /* MNG */
 		O_ICON
 	} type;
 	u_long fore;
@@ -375,14 +441,14 @@ struct render_object {
 		struct tfont *tfc;
 #endif /* FREETYPE */
 		struct {
-			u_char *xfont;
+			char *xfont;
 			u_int csize;
 			u_int code;
 			char *registry;
 		} xfont;
 		struct {
 			Image *image;
-			int xzoom, yzoom;
+			float xzoom, yzoom;
 		} image;
 		struct {
 			u_int itype;
@@ -390,6 +456,14 @@ struct render_object {
 			u_int npoint;
 			XPoint *xpoint;
 		} icon;
+#ifdef MNG
+		struct {
+			u_int width;
+			u_int height;
+			char *filename;
+			void *key;
+		} anim;
+#endif /* MNG */
 	} data;
 };
 
@@ -405,11 +479,11 @@ struct vfont {
 	u_int height;
 	u_int code;
 	u_int charlen;
+	u_int xmax;
 	int ascent;
 	int descent;
 	u_int xoff;
 	char *fontname;
-	XImage *image;
 	u_char *dbitmap;
 	u_int ref;
 };
@@ -427,13 +501,14 @@ struct tfont {
 	u_int height;		/* bitmap height */
 	u_int code;
 	u_int charlen;		/* origin x axis advance width */
+	u_int xmax;			/* right edge of bbox */
 	int ascent;		/* (top of bitmap) - (origin y axis) */
 	int descent;		/* (origin y axis) - (bottom of bitmap) */
 	int xoff;		/* (left of bitmap) - (origin x axis) */
 	char *fontname;
-	XImage *image;
 	u_char *dbitmap;
 	u_int ref;
+	int regid;		/* registry ID */
 };
 #endif /* FREETYPE */
 
@@ -442,12 +517,19 @@ struct alloc_color {
 	long	*colors;
 };
 
+struct bgpixmap {
+	struct ctrl *ctl;
+	Pixmap	pixmap;
+	Image *image;
+	XImageInfo *ximageinfo;
+};
+
 /*
  * The following variables are defined in global.c, and therefore
  * they are available in both "mgp" and "mgp2ps" binary.
  */
 #if 0
-extern u_char *page_data[MAXPAGE][MAXLINE];
+extern char *page_data[MAXPAGE][MAXLINE];
 #endif
 extern struct ctrl *page_control[MAXPAGE][MAXLINE];
 extern struct ctrl *default_control[MAXLINE];
@@ -470,24 +552,29 @@ extern int screen;
 extern int window_width;
 extern int window_height;
 extern Pixmap pixmap;
+extern Pixmap cachewin;
+extern Pixmap cachetmp;
+extern struct bgpixmap bgpixmap[MAXBGPIXMAP];
 extern Colormap colormap;
 extern struct alloc_color image_clr;
 extern struct alloc_color back_clr;
 extern struct alloc_color font_clr;
 
-extern u_int char_size;
-extern u_int horiz_gap;
-extern u_int vert_gap;
+extern u_int char_size[2];
+extern u_int horiz_gap[2];
+extern u_int vert_gap[2];
 extern u_int depth;
 extern Visual *visual;
-extern u_long fore_color;
-extern u_long back_color;
-extern u_long ctrl_color;
-extern u_int b_quality;
+extern u_long fore_color[2];
+extern u_long back_color[2];
+extern u_long ctrl_color[2];
+extern u_int b_quality[2];
 extern u_int quality_flag;
 
-#define VERT_GAP(s)		((s) * vert_gap / 100)
-#define HORIZ_GAP(s)		((s) * horiz_gap / 100)
+extern char mgp_charset[256];
+
+#define VERT_GAP(s)		((s) * vert_gap[caching] / 100)
+#define HORIZ_GAP(s)		((s) * horiz_gap[caching] / 100)
 #define VERT_STEP(s)		((s) + VERT_GAP(s))
 #define HORIZ_STEP(s, x)	((x) + HORIZ_GAP(s))
 
@@ -510,6 +597,8 @@ extern GC gc_plrev;
 extern GC gc_pta;
 extern GC gc_ptk;
 
+extern GC gc_cache;
+
 extern long xeventmask;
 
 /*
@@ -535,12 +624,16 @@ extern char *vfcap_name;
 #ifdef FREETYPE
 extern char *freetypefontdir;
 extern char *freetypefont0;	/* font name to be used as a last resort */
+extern char *freetypemfont0;	/* font name to be used as a last resort */
+extern int unicode_map[65536];
 #endif
+extern int latin_unicode_map[3][256];
 #ifdef TTY_KEYINPUT
 extern volatile int ttykey_enable;
 #endif
 extern char *back_clname;
 extern char *gsdevice;
+extern char *htmlimage;
 
 extern char *mgpwdir;
 extern char mgpwdirname[];
@@ -550,6 +643,7 @@ extern pid_t checkchild __P((void *));
 extern Window checkchildwin __P((void *));
 extern void regchild __P((pid_t, void *, Window, int));
 extern void purgechild __P((int));
+extern void remapchild __P((void));
 extern void cleanup __P((int));
 #ifdef TTY_KEYINPUT
 extern void try_enable_ttykey __P((void));
@@ -563,18 +657,19 @@ extern void state_next __P((struct render_state *));
 extern void draw_page __P((struct render_state *, struct ctrl *));
 extern Bool draw_one __P((struct render_state *, XEvent *));
 extern void timebar __P((struct render_state *));
-extern void draw_sizechange __P((struct render_state *));
+extern void draw_reinit __P((struct render_state *));
+extern int get_regid __P((char *));
 
 /*parse.c*/
-extern void load_file __P((u_char *));
+extern void load_file __P((char *));
 extern void cleanup_file __P(());
 extern int ctlcmp __P((struct ctrl *, struct ctrl *));
 extern FILE *fsearchopen __P((char *, char *, char **));
-extern int chkfile __P((u_char *));
+extern int chkfile __P((char *));
 extern struct ctrl *ctllastitem __P((struct ctrl *));
 extern void ctlappend __P((struct ctrl *, struct ctrl *));
 extern void ctlinsert __P((struct ctrl **, struct ctrl *));
-extern struct ctrl *ctlalloc1 __P((u_char));
+extern struct ctrl *ctlalloc1 __P((u_int));
 extern void ctlfree __P((struct ctrl *));
 extern struct ctrl *ctlcopy __P((struct ctrl *));
 extern void debug0 __P((struct ctrl *));
@@ -602,8 +697,9 @@ extern int window_y;
 extern void init_win1 __P((char *));
 extern void init_win2 __P((void));
 extern void init_win3 __P((void));
-extern int get_color __P((u_char *, u_long *));
-extern struct g_color *name2gcolor __P((u_char *));
+extern void finish_win __P((void));
+extern int get_color __P((char *, u_long *));
+extern struct g_color *name2gcolor __P((char *));
 extern void regist_alloc_colors __P((struct alloc_color *, u_long *, int));
 extern void free_alloc_colors __P((struct alloc_color *));
 
@@ -615,11 +711,12 @@ extern Image *make_XImage __P((byte *, unsigned int, unsigned int));
 
 /* postscript.c */
 extern int ps_boundingbox __P((char *, int *, int *, int *, int *));
+extern void image_zoomratio __P((struct render_state *, float *, float *, int, int, int));
 
 /* tfont.c */
 extern int tfc_setsize __P((u_int));
-extern struct tfont *tfc_get __P((u_int, u_int, int));
-extern void tfc_setfont __P((char *));
+extern struct tfont *tfc_get __P((u_int, u_int, int, char *, int));
+extern void tfc_setfont __P((char *, int, char *));
 extern XImage *tfc_image __P((struct tfont *, long, long, XImage *, int, int));
 
 /* embed.c */
@@ -644,3 +741,9 @@ extern void usleep __P((u_int));
 #ifdef VFLIB
 #define RASTERLIB
 #endif
+
+#if 1
+#define DITHERED_BGRAD
+#define COLOR_BUGFIX
+#endif
+
