@@ -26,7 +26,7 @@
  * SUCH DAMAGE.
  */
 /*
- * $Id: tfont.c,v 1.22 1998/09/04 21:21:45 itojun Exp $
+ * $Id: tfont.c,v 1.42 2001/04/12 11:49:04 nishida Exp $
  */
 
 #include "mgp.h"
@@ -44,12 +44,12 @@ static TT_Face			face[TFFONT_NUM];
 static TT_Glyph			glyph[TFFONT_NUM];
 static TT_Instance		instance[TFFONT_NUM];
 static TT_Glyph_Metrics		metrics;
-static TT_Outline		outline;
 static TT_Error			error;
 static TT_Engine		engine;
 static XImage *xim;
 
-static char *tf_curname = NULL;
+static char *tf_curname[4]; /* contains iso8859-[2-4] fonts */
+static char *tf_mcurname = NULL;
 static int tfcinitdone;
 static int tfcachecnt;
 static struct tfont tfclru;
@@ -59,6 +59,7 @@ static int tfcachehit = 0;
 static int tfcachemiss = 0;
 static int tfcuridx = -1;
 static char tfloadedfont[1024][TFFONT_NUM];
+static int tfloadedregistry[TFFONT_NUM];
 
 #define TFC_INSHASH(tfc) { \
 	struct tfont *h, *n;						\
@@ -84,9 +85,10 @@ static char tfloadedfont[1024][TFFONT_NUM];
 
 static void tfc_init __P((void));
 static void tfc_free __P((struct tfont *));
-static struct tfont *tfc_lookup __P((u_int, u_int, char*));
-static struct tfont *tfc_alloc __P((u_int, u_int, char *));
-static short CharToUnicode __P((u_int));
+static struct tfont *tfc_lookup __P((u_int, u_int, char*, int));
+static struct tfont *tfc_alloc __P((u_int, u_int, char *, char *));
+static short CharToUnicode __P((u_int, char *));
+static short jistosjis __P((u_int));
 static TT_Error LoadTrueTypeChar __P((int, int));
 
 #define TTFLOOR(x)	((x) & -64)
@@ -110,6 +112,11 @@ tfc_init()
 		tfc->next = tfc->prev = tfc;
 	tfclru.lrunext = tfclru.lruprev = &tfclru;
 	tfcinitdone ++;
+
+	latin_unicode_map_init();
+#ifdef FREETYPE_CHARSET16
+	unicode_map_init();
+#endif
 }
 
 static void
@@ -118,7 +125,6 @@ tfc_free(tfc)
 {
 	TFC_DELHASH(tfc);
 	TFC_DELLRU(tfc);
-	XDestroyImage(tfc->image);
 	free(tfc->fontname);
 	free(tfc->dbitmap);
 	free(tfc);
@@ -150,16 +156,27 @@ tfc_setsize(char_size)
 }
 
 struct tfont *
-tfc_get(code, size, force)
+tfc_get(code, size, force, registry, charset16)
 	u_int code, size;
 	int force;
+	char *registry;
+	int charset16;
 {
 	struct tfont *tfc, *ntfc;
+	int	regid;
 
 	if (!tfcinitdone)
 		tfc_init();
 
-	tfc = tfc_lookup(code, size, tf_curname);
+	if (charset16)
+		tfc = tfc_lookup(code, size, tf_mcurname, 0);
+	else {
+		if (code >= 0xa0 && ((!registry || !registry[0]) && mgp_charset)){
+			regid = get_regid(mgp_charset);
+		} else 
+			regid = get_regid(registry);
+		tfc = tfc_lookup(code, size, tf_curname[regid], regid);
+	}
 
 	if (tfc == NULL) {
 		/* XXX */
@@ -177,17 +194,22 @@ tfc_get(code, size, force)
 			}
 		}
 
-		tfc = tfc_alloc(code, size, tf_curname);
+		if (charset16)
+			tfc = tfc_alloc(code, size, tf_mcurname, registry);
+		else
+			tfc = tfc_alloc(code, size, tf_curname[regid], registry);
 	}
 	return tfc;
 }
 
 void
-tfc_setfont(name)
+tfc_setfont(name, charset16, registry)
 	char *name;
+	int charset16;  /*2-octet charset?*/
+	char *registry;
 {
-	char *fontname;
-	u_int i;
+	char *fontname = NULL;
+	u_int i, regid = 0;
 	int res = 96; /* XXX */
 	char pathname[MAXPATHLEN];
 	int trial;
@@ -212,8 +234,10 @@ tfc_setfont(name)
 			}
 			name = freetypefont0;
 		} else {
-			fprintf(stderr, "tfc_setfont: font name not given, "
-				"fail\n");
+			if (verbose) {
+				fprintf(stderr, "tfc_setfont: "
+					"font name not given, fail\n");
+			}
 			goto fail;
 		}
 	}
@@ -242,7 +266,12 @@ tfc_setfont(name)
 				}
 #endif
 				tfcuridx = i;
-				tf_curname = tfloadedfont[i];
+				if (charset16)
+					tf_mcurname = tfloadedfont[i];
+				else {
+					regid = get_regid(registry);
+					tf_curname[regid] = tfloadedfont[i];
+				}
 #if 0
 				tfc_setsize(char_size);
 #endif
@@ -266,14 +295,21 @@ tfc_setfont(name)
 			if (!freetypefont0 || !freetypefont0[0])
 				continue;
 			fontname = freetypefont0;
+			/* already loaded? */
+			for (i = 0; i < tffontcnt; i ++) {
+				if (!strcmp(fontname, tfloadedfont[i])){
+					tfcuridx = i;
+					if (verbose) 
+						fprintf(stderr, "font \"%s\" already loaded: %d\n", 
+							fontname, tffontcnt);
+					goto cached;
+				}
+			}
 			break;
 		}
 
 		/* Load face */
-		if (verbose) {
-			fprintf(stderr, "trying to open font \"%s\"\n",
-				fontname);
-		}
+		if (verbose) fprintf(stderr, "trying to open font \"%s\"\n", fontname);
 		error = TT_Open_Face(engine, fontname, &tface);
 		if (!error)
 			break;
@@ -291,7 +327,18 @@ tfc_setfont(name)
 		}
 		goto fail;
 	}
+	if (verbose) {
+		fprintf(stderr, "font \"%s\" opened successfully\n",
+			fontname);
+	}
 
+	/* get registry id */
+	if ((regid = get_regid(registry)) < 0){
+		fprintf(stderr, "font \"%s\" has irregal registry\n", fontname);
+			goto fail2;	
+	}
+
+	tfloadedregistry[tffontcnt] = regid;
 	strcpy(tfloadedfont[tffontcnt], fontname);
 	tfcuridx = tffontcnt;
 	face[tfcuridx] = tface;
@@ -328,8 +375,19 @@ tfc_setfont(name)
 		goto fail2;
 	}
 
-	tf_curname = tfloadedfont[tfcuridx];
-	if (tfc_setsize(char_size) < 0)
+cached:
+	if (charset16)
+		tf_mcurname = tfloadedfont[tfcuridx];
+	else {
+		if (registry)
+			tf_curname[regid] = tfloadedfont[tfcuridx];
+		else {
+			/* this should be default font */
+			for (i = 0; i < 4; i ++)
+				tf_curname[i] = tfloadedfont[tfcuridx];
+		}
+	}
+	if (tfc_setsize(char_size[caching]) < 0)
 		goto fail2;
 
 	/* success */
@@ -339,19 +397,26 @@ fail2:
 	TT_Close_Face(tface);
 	tffontcnt--;
 fail:
-	tf_curname = "";
-	tfcuridx = -1;
+	if (!charset16){
+		tf_curname[regid] = "";
+		tfcuridx = -1;
+	} else 
+		tf_mcurname = "";
 	return;
 }
 
 static struct tfont *
-tfc_lookup(code, size, fontname)
+tfc_lookup(code, size, fontname, regid)
 	u_int code, size;
 	char *fontname;
+	int	regid;
 {
 	u_int	i;
 	struct tfont *tfc, *htfc;
 
+	if (!fontname)  
+		return NULL;    
+
 	for (i = 0; i < tffontcnt; i ++) {
 		if (!strcmp(fontname, tfloadedfont[i]))
 			tfcuridx = i;
@@ -361,6 +426,7 @@ tfc_lookup(code, size, fontname)
 	for (tfc = htfc->next; tfc != htfc; tfc = tfc->next) {
 		if (tfc->code == code
 		 && tfc->size == size
+		 && tfc->regid == regid
 		 && strcmp(tfc->fontname, fontname) == 0) {
 			tfcachehit++;
 			TFC_DELLRU(tfc);
@@ -374,9 +440,10 @@ tfc_lookup(code, size, fontname)
 }
 
 static struct tfont *
-tfc_alloc(code, size, fontname)
+tfc_alloc(code, size, fontname, registry)
 	u_int code, size;
 	char *fontname;
+	char *registry;
 {
 	struct tfont *tfc;
 	short unicode;
@@ -385,7 +452,7 @@ tfc_alloc(code, size, fontname)
 	if (!tffontcnt) {
 		if (verbose)
 			fprintf(stderr, "no fontcount\n");
-		tfc_setfont(NULL);
+		tfc_setfont(NULL, 0, NULL);
 	}
 
 	/* font out of bounds, suggests no font was available */
@@ -395,8 +462,12 @@ tfc_alloc(code, size, fontname)
 		return NULL;
 	}
 
+	if (fontname == NULL)
+		return NULL;
+	
+
 	/* This is required! */
-	if (tfc_setsize(char_size) < 0)
+	if (tfc_setsize(char_size[caching]) < 0)
 		return NULL;
 
 	tfc = (struct tfont *)malloc(sizeof(*tfc));
@@ -406,7 +477,7 @@ tfc_alloc(code, size, fontname)
 	}
 
 #if 1
-	unicode = CharToUnicode(code);
+	unicode = CharToUnicode(code, registry);
 #else
 	unicode = code;
 #endif
@@ -460,11 +531,13 @@ tfc_alloc(code, size, fontname)
 		TTCEIL(metrics.bbox.yMax - metrics.bbox.yMin - metrics.bearingY) >> 6;
 	tfc->ascent = bitmap.rows - tfc->descent;
 	tfc->xoff = metrics.bearingX >> 6;
-	tfc->fontname = strdup(tf_curname);
+	tfc->fontname = strdup(fontname);
 	tfc->charlen = metrics.advance >> 6;
+	tfc->xmax = metrics.bbox.xMax >> 6;
 	tfc->ref = 0;
 	tfc->dbitmap = bitmap.bitmap;
-	tfc->image = xim;
+	tfc->regid = get_regid(registry);
+	if (!tfc->charlen) tfc->charlen = 1;
 
 	TFC_INSHASH(tfc);
 	TFC_INSLRU(tfc);
@@ -474,12 +547,15 @@ tfc_alloc(code, size, fontname)
 }
 
 static short
-CharToUnicode(code)
+CharToUnicode(code, registry)
 	u_int code;
+	char *registry;
 {
 	TT_CharMap      char_map;
 	unsigned short  i, n;
 	unsigned short  platform, encoding;
+	unsigned int	regid;
+	char			p;
 
 	/* First, look for a Unicode charmap */
 #ifdef HAVE_TT_FACE_PROPERTIES_CHARMAPS
@@ -488,13 +564,65 @@ CharToUnicode(code)
 	n = TT_Get_CharMap_Count(face[tfcuridx]);
 #endif
 
+#if 1
+	if (verbose) {
+		fprintf(stderr, "%d charmaps for font %s code %04x\n",
+			n, tfloadedfont[tfcuridx], code & 0xffff);
+	}
+#endif
+
 	for (i = 0; i < n; i++) {
 		TT_Get_CharMap_ID(face[tfcuridx], i, &platform, &encoding);
 		if ((platform == 3 && encoding == 1)
 		 || (platform == 0 && encoding == 0)) {
 			TT_Get_CharMap(face[tfcuridx], i, &char_map);
+#ifdef FREETYPE_CHARSET16
+			if (code > 256 && strncmp(registry, "jisx0208.1983-", 14) != 0) {
+				/*
+				 * unicode_map assumes JIS X0208.
+				 * it is not usable for other encodings.
+				 * try the next CharMap available.
+				 */
+				continue;
+			}
+			if (code > 256)
+				code = unicode_map[code];
+#endif /* FREETYPE_CHARSET16 */
+
+#if 1 /* latin2-4 encoding processing */
+			if (code > 0xa0) {
+				if (registry)
+					regid = get_regid(registry) -1;
+				else {
+					if (mgp_charset)
+						regid = get_regid(mgp_charset) -1;
+				}
+				if (latin_unicode_map[regid][code])
+					code = latin_unicode_map[regid][code];
+			}
+#endif
+
 			return TT_Char_Index(char_map, (short) code);
 		}
+#ifdef FREETYPE_CHARSET16
+		else {
+			/*
+			 * very adhoc processing for truetype fonts generated by cb2ttj.
+			 */
+			if ((platform == 3 && encoding == 2)
+		 		|| (platform == 1 && encoding == 1)) {
+					if (code < 256) continue;
+					TT_Get_CharMap(face[tfcuridx], i, &char_map);
+					return TT_Char_Index(char_map, (u_short) jistosjis(code));
+			} else {
+				if (platform == 1 && encoding == 0) {
+					if (code >= 0xa0) continue;
+					TT_Get_CharMap(face[tfcuridx], i, &char_map);
+					return TT_Char_Index(char_map, (short) code);
+				}
+			}
+		}
+#endif
 	}
 
 	fprintf(stderr, "Sorry, but the fontfile \"%s\" doesn't "
@@ -606,30 +734,44 @@ tfc_image(tfc, fore, back, xim, bx, by)
 			}
 		}
 	}
-#ifdef GLYPHEDGE
-	/*
-	 * for debugging treatment of font metrics, for 16bpp displays
-	 */
-	/* pixmap bounding box */
-	for (y = 0; y < tfc->height; y++) {
-		XPutPixel(xim, bx + tfc->xoff, by - tfc->ascent + y, 0xffff);
-		XPutPixel(xim, bx + tfc->xoff + tfc->width - 1,
-			by - tfc->ascent + y, 0xffff);
+	if (mgp_flag & FL_GLYPHEDGE) {
+		/*
+		 * for debugging treatment of font metrics, for 16bpp displays
+		 */
+		/* pixmap bounding box */
+		for (y = 0; y < tfc->height; y++) {
+			XPutPixel(xim, bx + tfc->xoff, by - tfc->ascent + y,
+				0xffff);
+			XPutPixel(xim, bx + tfc->xoff + tfc->width - 1,
+				by - tfc->ascent + y, 0xffff);
+		}
+		for (x = 0; x < tfc->width; x++) {
+			XPutPixel(xim, bx + tfc->xoff + x, by - tfc->ascent,
+				0xffff);
+			XPutPixel(xim, bx + tfc->xoff + x,
+				by - tfc->ascent + tfc->height - 1, 0xffff);
+		}
+		/* origin */
+		XPutPixel(xim, bx, by, 0xaaaa);
+		/* baseline */
+		for (x = 0; x < tfc->width; x++)
+			XPutPixel(xim, bx + tfc->xoff + x, by, 0xaaaa);
 	}
-	for (x = 0; x < tfc->width; x++) {
-		XPutPixel(xim, bx + tfc->xoff + x, by - tfc->ascent, 0xffff);
-		XPutPixel(xim, bx + tfc->xoff + x,
-			by - tfc->ascent + tfc->height - 1, 0xffff);
-	}
-	/* origin */
-	XPutPixel(xim, bx, by, 0xaaaa);
-	/* baseline */
-	for (x = 0; x < tfc->width; x++)
-		XPutPixel(xim, bx + tfc->xoff + x, by, 0xaaaa);
-#endif
 	if (bcol)
 		free(bcol);
 	return xim;
 }
 
+static short
+jistosjis(code)
+	u_int code;
+{
+	u_char c1 = code >> 8;
+	u_char c2 = code % 256;
+	int rowOffset = c1 < 95 ? 112: 176;
+	int cellOffset = c1 % 2 ? (c2 > 95 ? 32 : 31) : 126;
+
+	return (((c1 + 1) >> 1) + rowOffset) * 256 + c2 + cellOffset;
+}
+
 #endif /* FRRETYPE */
